/*
 * Copyright (C) 1994
 *
 * Department of Computing Science,
 * The University,
 * Newcastle upon Tyne,
 * UK.
 */

#ifndef PROCESSHEAP_H_
#define PROCESSHEAP_H_

#ifndef BOOLEAN_H_
#  include "Boolean.h"
#endif

#ifndef PROCESS_H_
#  include "Process.h"
#endif

#ifndef LISTBASE_H_
#  include "ListBase.h"
#endif


class HeapElement;
class ostream;


/*
 * A heap-based queue for maintaining simulation processes.
 * This is an implementation of the scheme described in the
 * C++SIM paper.
 * We keep the number of event lists to a minimum by comparing
 * approximate times (event times as a long rather than as a
 * double).
 * Note: due to an oversight, this was not shipped in versions
 * prior to 1.5.
 */

class ProcessHeap : public ListBase
{
    friend class HeapIterator;
public:
    ProcessHeap ();
    ~ProcessHeap ();

    void Insert (Process&, Boolean prior = FALSE);
    Boolean InsertBefore (Process &ToInsert, Process &Before);
    Boolean InsertAfter (Process &ToInsert, Process &After);
    Process *Remove (const Process *p = 0);
    const Process* getNext (const Process*) const;

    std::ostream& print (std::ostream&) const;

private:
    Process* findMinimum () const;

    void InsertAt (Process&, HeapElement*);
    void RemoveFrom (HeapElement*);

    HeapElement* Head;
    int numberOfElements;
};

extern std::ostream& operator<< (std::ostream& strm, const ProcessHeap& ph);

#endif
