/*
 * Copyright (C) 1994
 *
 * Department of Computing Science,
 * The University,
 * Newcastle upon Tyne,
 * UK.
 */

#include <errno.h>
#include <iostream>
#include <fstream>
//#include <values.h>
#include <iomanip>

#ifndef ERROR_H_
#  include "Error.h"
#endif

#ifndef MEAN_H_
#  include "Mean.h"
#endif

// Added by Jacky Luk in place of values.h
#define MAXFLOAT ((float) 3.40e+38)

Mean::Mean () { Mean::reset(); }

Mean::~Mean () {}

void Mean::reset ()
{
    _Max = -MAXFLOAT;
    _Min = MAXFLOAT;
    _Sum = _Mean = 0.0;
    _Number = 0;
}

void Mean::setValue (double value)
{
    if (value > _Max)
        _Max = value;
    if (value < _Min)
        _Min = value;
    _Sum += value;
    _Number++;
    _Mean = (double) (_Sum/_Number);
}

Boolean Mean::saveState (std::ofstream& oFile) const
{
    if (!oFile)
	return FALSE;
    
    oFile << " " << _Max << " " << _Min;
    oFile << " " << _Sum << " " << _Mean;
    oFile << " " << _Number << " ";
    return TRUE;
}

Boolean Mean::saveState (const char* fileName) const
{
    std::ofstream oFile(fileName, std::ios::out);
    Boolean result;
    
    if (!oFile)
    {
	error_stream << WARNING << "Mean::saveState - error " << errno << " for file " << fileName << std::endl;
	return FALSE;
    }
    
    result = saveState(oFile);
    oFile.close();
    return result;
}

Boolean Mean::restoreState (std::ifstream& iFile)
{
    if (!iFile)
	return FALSE;
    
    iFile >> _Max >> _Min;
    iFile >> _Sum >> _Mean;
    iFile >> _Number;
    return TRUE;
}

Boolean Mean::restoreState (const char* fileName)
{
    std::ifstream iFile(fileName, std::ios::in);
    Boolean result;
    
    if (!iFile)
    {
	error_stream << WARNING << "Mean::restoreState - error " << errno << " for file " << fileName << std::endl;
	return FALSE;
    }
    
    result = restoreState(iFile);
    iFile.close();
    return result;
}

std::ostream& Mean::print (std::ostream& strm) const
{
    strm << "Number of samples : " << numberOfSamples() << std::endl;
    strm << "Minimum           : " << min() << std::endl;
    strm << "Maximum           : " << max() << std::endl;
    strm << "Sum               : " << sum() << std::endl;
    strm << "Mean              : " << mean() << std::endl;

    return strm;
}

#ifdef NO_INLINES
#  define MEAN_CC_
#  include "Mean.n"
#  undef MEAN_CC_
#endif
